@@ -1,10 +1,10 @@
 package Module::Install::DOAPChangeSets;
 
 use 5.008;
-use base qw(Module::Install::Base);
+use parent qw(Module::Install::Base);
 use strict;
 
-our $VERSION = '0.201';
+our $VERSION = '0.203';
 our $AUTHOR_ONLY = 1;
 
 sub write_doap_changes {
