@@ -1,23 +1,3 @@
-#!/usr/bin/perl
-
-=head1 NAME
-
-RDF::DOAP::ChangeSets - create pretty ChangeLogs from RDF
-
-=head1 SYNOPSIS
-
- use RDF::DOAP::ChangeSets;
- use URI::file;
- 
- my $file     = 'path/to/changelog.rdf';
- my $file_uri = URI::file->new_abs($file);
- 
- my $dcs = RDF::DOAP::ChangeSets->new(
-             $file_uri, undef, undef, 'RDFXML');
- print $dcs->to_string;
-
-=cut
-
 package RDF::DOAP::ChangeSets;
 
 use 5.008;
@@ -30,39 +10,7 @@ use RDF::Trine;
 use RDF::Query;
 use Text::Wrap;
 
-our $VERSION = '0.201';
-
-=head1 DESCRIPTION
-
-This module takes software changelogs written in the RDF DOAP
-Change Sets vocabulary and produces human-readable changelogs.
-
-=over
-
-=item C<< RDF::DOAP::ChangeSets->new($uri, $data, $type, $fmt) >>
-
-Creates and initialises an object.
-
-$uri is a URL for the input data. The URL is used to query the
-RDF data for the heading of the output changelog. It may be passed
-as either a string, or a L<URI> object.
-
-$data is the RDF data to use as input. It may be passed as a
-string, or as an L<RDF::Trine::Model> object. If undefined,
-this module will attempt to read data from the URL using
-L<LWP::Simple>.
-
-$type gives the constructor a hint as to the RDF vocabulary you
-are using. For DOAP Change Sets, use 'current'; for Aaron Cope's
-Changefile vocab, use 'legacy'; to autodetect, use 'auto'. By
-default, performs autodetection. This module may crash and burn
-if you try to mix both vocabs!!
-
-$fmt provides a hint as to what RDF format you're using. By
-default, Turtle is assumed. Valid values are whatever
-RDF::Trine::Parser->new accepts.
-
-=cut
+our $VERSION = '0.203';
 
 sub new
 {
@@ -117,60 +65,30 @@ sub new
 	bless $self, $class;
 }
 
-=item C<< $changeset->is_legacy >>
-
-Boolean, indicating if a legacy vocab is being used.
-
-=cut
-
 sub is_legacy
 {
 	my $self = shift;
 	return (lc $self->{'type'} eq 'legacy');
 }
 
-=item C<< $changeset->is_current >>
-
-Boolean, indicating if the current vocab is being used.
-
-=cut
-
 sub is_current
 {
 	my $self = shift;
 	return !$self->is_legacy(@_);
 }
 
-=item C<< $changeset->model >>
-
-RDF::Trine::Model object representing the changelog data.
-
-=cut
-
 sub model
 {
 	my $self = shift;
 	return $self->{'model'};
 }
 
-=item C<< $changeset->uri >>
-
-String representing the changelog URI.
-
-=cut
-
 sub uri
 {
 	my $self = shift;
 	return $self->{'uri'} . '';
 }
 
-=item C<< $changeset->to_string >>
-
-Creates a human-readable representation of the changelog.
-
-=cut
-
 sub to_string
 {
 	my $self = shift;
@@ -232,30 +150,43 @@ sub to_string
 			$rv.= $version->{'revision'};
 			$rv.= sprintf('  %s', $version->{'issued'})
 				if $version->{'issued'};
-			$rv.= sprintf("\n# %s", $version->{'name'})
+			$rv.= sprintf("  # %s", $version->{'name'})
 				if $version->{'name'};
 			$rv.= "\n\n";
 			
-			my @changes = sort {
-				$a->{type} cmp $b->{type} or $a->{label} cmp $b->{label}
-				} values %{$version->{'c'}};
-			
-			# foreach change
-			foreach my $change (@changes)
-			{
-				my $sigil = '';
-				if (defined $change->{'type'} and ref($change->{'type'}) eq 'ARRAY')
+			my @changes = map
 				{
-					$sigil = join ' ',
-						sort
-						map { m!doap.changeset.(.+)$!; $1; }
-						grep { m!doap.changeset.(.+)$! }
-						@{ $change->{'type'} };
-					$sigil = "(${sigil}) " if length $sigil;
+					my $change = $_;
+					
+					my $sigil = '';
+					if (defined $change->{'type'} and ref($change->{'type'}) eq 'ARRAY')
+					{
+						$sigil = join ' ',
+							sort
+							map { m!doap.changeset.(.+)$!; $1; }
+							grep { m!doap.changeset.(.+)$! }
+							@{ $change->{'type'} };
+						$sigil = "(${sigil}) " if length $sigil;
+					}
+					# Bullet point
+					my $ret = wrap(' - ', '   ', sprintf("%s%s", $sigil, $change->{'label'})) . "\n";
+					
+					my %blame = %{ $change->{blame} || {}};
+					foreach $b (values %blame)
+					{
+						if (defined $b->{nick})
+							{ $ret .= sprintf("   ++\$%s\n", $b->{nick}) }
+						elsif (defined $b->{name})
+							{ $ret .= sprintf("   ++\"%s\"\n", $b->{name}) }
+						elsif (defined $b->{uri})
+							{ $ret .= sprintf("   ++q<%s>\n", $b->{uri}) }
+					}
+					
+					$ret;
 				}
-				# Bullet point
-				$rv.= wrap(' - ', '   ', sprintf("%s%s", $sigil, $change->{'label'})) . "\n";
-			}
+				values %{$version->{'c'}};
+			
+			$rv.= join q{}, sort @changes;
 			$rv.= "\n";
 		}
 		
@@ -264,12 +195,6 @@ sub to_string
 	return $rv;
 }
 
-=item C<< $changeset->to_file($filename) >>
-
-Same as C<to_string>, but outputs to a file.
-
-=cut
-
 sub to_file
 {
 	my $self = shift;
@@ -489,6 +414,12 @@ sub _release_data__current
 			?version dcs:changeset [ dcs:item ?item ] .
 			OPTIONAL { ?item a ?itemtype . }
 			OPTIONAL { ?item rdfs:label ?itemlabel . }
+			OPTIONAL {
+				?item dcs:blame|dcs:thanks ?blame .
+				OPTIONAL { ?blame foaf:nick ?blamenick . }
+				OPTIONAL { ?blame foaf:name ?blamename . }
+				OPTIONAL { ?blame rdfs:label ?blamename . }
+			}
 		}
 	}
 	";
@@ -515,7 +446,18 @@ sub _release_data__current
 			push @{ $projects->{$p}->{'v'}->{$v}->{'c'}->{$c}->{'type'} }, $row->{'itemtype'}->uri
 				if UNIVERSAL::isa($row->{'itemtype'}, 'RDF::Trine::Node::Resource')
 				and $row->{'itemtype'}->uri ne 'http://ontologi.es/doap-changeset#Change';
-		}
+			$projects->{$p}->{'v'}->{$v}->{'c'}->{$c}->{'blame'} = {};
+			
+			if (UNIVERSAL::isa($row->{'blame'}, 'RDF::Trine::Node'))
+			{
+				my $b = $row->{'blame'}->as_ntriples;
+				$projects->{$p}->{'v'}->{$v}->{'c'}->{$c}->{'blame'}->{$b} = {
+						uri  => $row->{'blame'}->is_resource ? $row->{'blame'}->uri : undef,
+						name => $row->{'blamename'}&&$row->{'blamename'}->is_literal ? $row->{'blamename'}->literal_value : undef,
+						nick => $row->{'blamenick'}&&$row->{'blamenick'}->is_literal ? $row->{'blamenick'}->literal_value : undef,
+					};
+			}
+		}		
 	}
 }
 
@@ -562,6 +504,7 @@ sub _release_data__legacy
 			my $c = $row->{'itemlabel'}->as_ntriples;
 			$projects->{$p}->{'v'}->{$v}->{'c'}->{$c}->{'label'} = $row->{'itemlabel'}->literal_value
 				if UNIVERSAL::isa($row->{'itemlabel'}, 'RDF::Trine::Node::Literal');
+			$projects->{$p}->{'v'}->{$v}->{'c'}->{$c}->{'blame'} = {};
 				
 			if (UNIVERSAL::isa($row->{'itemtype'}, 'RDF::Trine::Node::Resource'))
 			{
@@ -581,6 +524,76 @@ sub _release_data__legacy
 
 __END__
 
+=head1 NAME
+
+RDF::DOAP::ChangeSets - create pretty ChangeLogs from RDF
+
+=head1 SYNOPSIS
+
+ use RDF::DOAP::ChangeSets;
+ use URI::file;
+ 
+ my $file     = 'path/to/changelog.rdf';
+ my $file_uri = URI::file->new_abs($file);
+ 
+ my $dcs = RDF::DOAP::ChangeSets->new(
+             $file_uri, undef, undef, 'RDFXML');
+ print $dcs->to_string;
+
+=head1 DESCRIPTION
+
+This module takes software changelogs written in the RDF DOAP
+Change Sets vocabulary and produces human-readable changelogs.
+
+=over
+
+=item C<< RDF::DOAP::ChangeSets->new($uri, $data, $type, $fmt) >>
+
+Creates and initialises an object.
+
+$uri is a URL for the input data. The URL is used to query the
+RDF data for the heading of the output changelog. It may be passed
+as either a string, or a L<URI> object.
+
+$data is the RDF data to use as input. It may be passed as a
+string, or as an L<RDF::Trine::Model> object. If undefined,
+this module will attempt to read data from the URL using
+L<LWP::Simple>.
+
+$type gives the constructor a hint as to the RDF vocabulary you
+are using. For DOAP Change Sets, use 'current'; for Aaron Cope's
+Changefile vocab, use 'legacy'; to autodetect, use 'auto'. By
+default, performs autodetection. This module may crash and burn
+if you try to mix both vocabs!!
+
+$fmt provides a hint as to what RDF format you're using. By
+default, Turtle is assumed. Valid values are whatever
+RDF::Trine::Parser->new accepts.
+
+=item C<< $changeset->is_legacy >>
+
+Boolean, indicating if a legacy vocab is being used.
+
+=item C<< $changeset->is_current >>
+
+Boolean, indicating if the current vocab is being used.
+
+=item C<< $changeset->model >>
+
+RDF::Trine::Model object representing the changelog data.
+
+=item C<< $changeset->uri >>
+
+String representing the changelog URI.
+
+=item C<< $changeset->to_string >>
+
+Creates a human-readable representation of the changelog.
+
+=item C<< $changeset->to_file($filename) >>
+
+Same as C<to_string>, but outputs to a file.
+
 =back
 
 =head1 BUGS
@@ -599,7 +612,7 @@ Toby Inkster E<lt>tobyink@cpan.orgE<gt>.
 
 =head1 COPYRIGHT
 
-Copyright 2010-2011 Toby Inkster
+Copyright 2010-2012 Toby Inkster
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
